package pwc.taxtech.atms.controller;

import org.apache.http.HttpStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import pwc.taxtech.atms.dto.PagingResultDto;
import pwc.taxtech.atms.dto.vatdto.InputInvoicePreviewQueryParam;
import pwc.taxtech.atms.dto.vatdto.InputVATInvoiceBaseDto;
import pwc.taxtech.atms.dto.vatdto.InputVATInvoiceDto;
import pwc.taxtech.atms.thirdparty.ExcelSheet;
import pwc.taxtech.atms.thirdparty.ExcelUtil;
import pwc.taxtech.atms.vat.entity.InputVATInvoiceItem;
import pwc.taxtech.atms.vat.service.impl.InputInvoiceDataImportServiceImpl;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

@RestController
@RequestMapping(value = "api/v1/inputInvoiceImport")
public class InputInvoiceImportController {
    @Autowired
    InputInvoiceDataImportServiceImpl inputInvoiceDataImportService;

    @RequestMapping(value = "inputInvoicePreviewList", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    PagingResultDto<InputVATInvoiceDto> getInputInvoiceTreeViewData(@RequestBody InputInvoicePreviewQueryParam paras) {
        return inputInvoiceDataImportService.getInputInvoiceTreeViewData(paras);
    }

    @RequestMapping(value = "exportQueryData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadInvoiceQueryData(@RequestBody InputInvoicePreviewQueryParam paras, HttpServletResponse response) {
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + UUID.randomUUID() + ".xlsx");
        response.setHeader("x-file-name", UUID.randomUUID() + ".xlsx");

        OutputStream os = null;
        try {
            os = response.getOutputStream();
            int count = getDownloadFilePath(paras, os);

            if (count == 0) {
                response.setStatus(HttpStatus.SC_NO_CONTENT);
            } else {
                response.setStatus(HttpStatus.SC_OK);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    @RequestMapping(value = "getInputInvoiceList/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity getInputInvoiceList(@PathVariable Integer period) {
        return ResponseEntity.ok().body(inputInvoiceDataImportService.getInputInvoiceList(period));
    }

    @RequestMapping(value = "getInputInvoiceItemList/{period}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public ResponseEntity getInputInvoiceItemList(@PathVariable Integer period) {
        return ResponseEntity.ok().body(inputInvoiceDataImportService.getInputInvoiceItemList(period));
    }

    private int getDownloadFilePath(InputInvoicePreviewQueryParam paras, OutputStream outputStream) {
        List<InputVATInvoiceDto> list = inputInvoiceDataImportService.getInputInvoiceTreeViewData(paras).getList();
        if (list.size() == 0) {
            return 0;
        }
        Map<String, String> header = new HashMap<>();
        header.put("PeriodID", "期间");
        header.put("InvoiceDate", "开票日期");
        header.put("InvoiceCode", "发票代码");
        header.put("InvoiceNumber", "发票号码");
        header.put("SellerTaxNumber", "供货方税号");
        header.put("InvoiceTypeName", "发票类型");
        header.put("Amount", "金额");
        header.put("TaxAmount", "税额");
        header.put("CertificationDate", "认证日期");

        ExcelSheet excelSheetA = new ExcelSheet<>();
        excelSheetA.setHeaders(header);
        excelSheetA.setDataset(list);
        excelSheetA.setSheetName("sheetA");


        Map<String, String> header2 = new HashMap<>();
        header2.put("PeriodID", "期间");
        header2.put("InvoiceCode", "发票代码");
        header2.put("InvoiceNumber", "发票号码");
        header2.put("Amount", "金额");
        header2.put("TaxAmount", "税额");
        header2.put("TaxRate", "税率");
        header2.put("ProductionName", "货物名称");

        ExcelSheet excelSheetB = new ExcelSheet<>();
        excelSheetB.setHeaders(header2);
        excelSheetB.setSheetName("sheetB");

        List<InputVATInvoiceItem> inputInvoiceItemList = new ArrayList<>();
        for (InputVATInvoiceDto item : list) {
            inputInvoiceItemList.addAll(inputInvoiceDataImportService.getInputInvoiceItemTreeViewData(item.getID()));
        }
        excelSheetB.setDataset(inputInvoiceItemList);
        List<ExcelSheet<InputVATInvoiceBaseDto>> sheets = new ArrayList<>();
        sheets.add(excelSheetA);
        sheets.add(excelSheetB);

        ExcelUtil.exportExcel(sheets, outputStream);
        return list.size();
    }
}
