package pwc.taxtech.atms.service.impl;

import com.beust.jcommander.internal.Lists;
import com.google.common.base.Predicate;
import org.apache.ibatis.io.Resources;
import org.apache.ibatis.jdbc.ScriptRunner;
import org.apache.ibatis.session.SqlSession;
import org.mybatis.spring.SqlSessionTemplate;
import org.reflections.util.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.agent.DatabaseAgent;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.OperateLogType;
import pwc.taxtech.atms.common.OperationModule;
import pwc.taxtech.atms.common.datasource.ShardingContextHolder;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.dao.OrganizationServiceTemplateGroupMapper;
import pwc.taxtech.atms.dao.ProjectClientMapper;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dao.ProjectServiceTypeMapper;
import pwc.taxtech.atms.dao.ServiceTypeMapper;
import pwc.taxtech.atms.dao.StandardAccountMapper;
import pwc.taxtech.atms.dao.TemplateGroupMapper;
import pwc.taxtech.atms.dto.FieldsMapper;
import pwc.taxtech.atms.dto.ProjectClientDto;
import pwc.taxtech.atms.dto.ServiceTypeDto;
import pwc.taxtech.atms.dto.TemplateGroupDto;
import pwc.taxtech.atms.dto.taxadmin.AddProjectResult;
import pwc.taxtech.atms.dto.taxadmin.ProjectDisplayDto;
import pwc.taxtech.atms.dto.taxadmin.ProjectYearParam;
import pwc.taxtech.atms.entitiy.*;
import pwc.taxtech.atms.entitiy.ProjectClientExample.Criteria;
import pwc.taxtech.atms.service.OperationLogService;
import pwc.taxtech.atms.service.ProjectService;
import pwc.taxtech.atms.vat.dao.VatStandardAccountMapper;
import pwc.taxtech.atms.vat.entity.VatStandardAccount;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

import static pwc.taxtech.atms.constant.Constant.*;

@Service
public class ProjectServiceImpl implements ProjectService {
    /**
     * @see PwC.Tax.Tech.Atms..Admin.Application\Services\impl\ProjectService.cs
     */
    private static final int FIRST_OR_DEFAULT = 0;
    private static Logger LOGGER = LoggerFactory.getLogger(ProjectServiceImpl.class);

    @Autowired
    private ServiceTypeMapper serviceTypeMapper;

    @Autowired
    private TemplateGroupMapper templateGroupMapper;

    @Autowired
    private ProjectClientMapper projectClientMapper;

    @Autowired
    private ProjectMapper projectMapper;

    @Autowired
    private OrganizationServiceTemplateGroupMapper organizationServiceTemplateGroupMapper;

    @Autowired
    private ProjectServiceTypeMapper projectServiceTypeMapper;

    @Autowired
    private OperationLogService operationLogService;

    @Autowired
    private DatabaseAgent databaseAgent;

    @Autowired
    private SqlSessionTemplate dynamicSqlSessionTemplate;

    @Autowired
    private StandardAccountMapper standardAccountMapper;

    @Autowired
    private VatStandardAccountMapper vatStandardAccountMapper;

    @Value("${jdbc_admin_db}")
    private String adminDbName;

    private final OperateLogType LOG_TYPE = OperateLogType.OperationLogProject;
    private static final Logger logger = LoggerFactory.getLogger(ProjectServiceImpl.class);

    @Override
    public List<ServiceTypeDto> getServiceList() {

        ServiceTypeExample serviceTypeExample = new ServiceTypeExample();
        serviceTypeExample.createCriteria().andIsActiveEqualTo(true);

        logger.debug("获取ServiceType Start");
        List<ServiceType> serviceTypeList = serviceTypeMapper.selectByExample(serviceTypeExample);

        logger.debug("获取TemplateGroup Start");
        List<TemplateGroup> templateGroupList = templateGroupMapper.selectByExample(new TemplateGroupExample());

        List<ServiceTypeDto> serviceTypeDtoList = serviceTypeList.stream().map(this::rotateServiceType)
                .collect(Collectors.toList());
        List<TemplateGroupDto> templateGroupDtoList = templateGroupList.stream().map(this::rotateTemplateGroup)
                .collect(Collectors.toList());

        for (ServiceTypeDto serviceTypeDto : serviceTypeDtoList) {
            // serviceType 和 templateGroup是上下层级关系，遍历ServiceTypeDto,
            // 并将每个ServiceType下级关联的templateGroup填充进去
            serviceTypeDto.setTemplateGroupList(templateGroupDtoList.stream()
                    .filter(templateGroupDto -> (serviceTypeDto.getID().equals(templateGroupDto.getServiceTypeId())))
                    .collect(Collectors.toList()));
        }
        return serviceTypeDtoList;
    }

    @Override
    public List<ProjectClientDto> getProjectClientList() {
        logger.debug("获取ProjectClient Start");
        ProjectClientExample example = new ProjectClientExample();
        example.setOrderByClause("CreateTime desc");
        Criteria criteria = example.createCriteria();
        criteria.andIsActiveEqualTo(true);
        List<ProjectClient> projectClientList = projectClientMapper.selectByExample(example);
        return projectClientList.stream().map(this::rotateProjectClient).collect(Collectors.toList());
    }

    /**
     * @TODO: this dmeo only for running,should change to query data from db (neo)
     */
    @Override
    public List<ProjectDisplayDto> getProjectByID(String projectID) {
        List<ProjectDisplayDto> displayDtos = new ArrayList<>();
        displayDtos.add(new ProjectDisplayDto().demo());
        return displayDtos;
    }

    @Override
    public List<ProjectDisplayDto> getAllProjectList(String orgID, String serviceID, Integer projectYear) {
        List<ProjectDisplayDto> pList = new ArrayList<>();
        List<ProjectDisplayDto> data = projectMapper.getProjectList(orgID, serviceID, projectYear);
        //获取每个项目的项目状态
        data.forEach(p -> {
            List<ProjectStatusManage> manageStatus = projectMapper.getProjectSatusListByDbName(p.getDbName());
            Map<Integer, Integer> dic = new HashMap<>();
            manageStatus.forEach(m -> {
                dic.put(m.getPeriodId(), m.getStatus());
            });

            p.setProjectStatusList(dic);
        });
        //添加账套中需要创建的项目
        List<ProjectDisplayDto> orgList = getProjectFromEnterpriseAccountSetOrg(data, orgID, serviceID, projectYear);
        //修改账套时间变化引起的已建项目的月份更改
        List<ProjectDisplayDto> removeList = new ArrayList<>();

        orgList.forEach(p -> {
            List<ProjectDisplayDto> fixList = data.stream()
                    .filter((Predicate<ProjectDisplayDto>) x -> x.getOrganizationID().equals(p.getOrganizationID())
                            && x.getEnterpriseAccountSetID().equals(p.getEnterpriseAccountSetID())
                            && x.getServiceTypeID().equals(p.getServiceTypeID())
                            && x.getYear().equals(p.getYear())
                            && (!x.getStartPeriod().equals(p.getStartPeriod()) || x.getEndPeriod().equals(p.getEndPeriod())))
                    .collect(Collectors.toList());

            if (fixList != null && !fixList.isEmpty()) {
                ProjectDisplayDto pddFirst = fixList.get(FIRST_OR_DEFAULT);

                //修改数据库对应项目起始月
                Project project = new Project();
                project.setID(pddFirst.getId());
                project.setStartPeriod(p.getStartPeriod());
                project.setEndPeriod(p.getEndPeriod());

                projectMapper.updateByPrimaryKey(project);

                //修改已取出数据中项目起始月
                pddFirst.setStartPeriod(p.getStartPeriod());
                pddFirst.setEndPeriod(p.getEndPeriod());
                //删除该条为创建项目记录
                removeList.add(p);
            }
        });

        removeList.forEach(r -> {
            orgList.remove(r);
        });

        pList.addAll(data);
        pList.addAll(orgList);
        return pList;
    }

    @Override
    public AddProjectResult addProject(ProjectDisplayDto projectDto, String userName) {
        projectDto.setActive(true);
        projectDto.setRuleType(1);

        Project project = new Project();
        try {
            FieldsMapper.map(projectDto, project);
        } catch (Exception e) {
            LOGGER.warn("map project dto to project may be some error");
            e.printStackTrace();
        }

        try {
            project.setID(UUID.randomUUID().toString());

            OrganizationServiceTemplateGroupExample example = new OrganizationServiceTemplateGroupExample();
            example.createCriteria().andOrganizationIdEqualTo(projectDto.getOrganizationID());
            organizationServiceTemplateGroupMapper.selectByExample(example).forEach(ostg -> {
                ProjectServiceType pst = new ProjectServiceType();
                pst.setID(UUID.randomUUID().toString());
                pst.setProjectID(project.getID());
                pst.setServiceTypeID(ostg.getServiceTypeId());
                pst.setTemplateGroupID(ostg.getTemplateGroupId());

                projectServiceTypeMapper.insert(pst);
                operationLogService.addDataAddLog(pst, OperationModule.ProjectServiceType, userName,
                        Message.Project.AddProjectServiceType, pst.getID(), projectDto.getName(), LOG_TYPE);
            });

            project.setDbName(generateDatabase(project, projectDto.getEnterpriseAccountSetID()));
            project.setUpdateTime(new Date());
            projectMapper.insert(project);
            operationLogService.addDataAddLog(project, OperationModule.Project, userName, Message.Project.AddProject,
                    project.getID(), projectDto.getName(), LOG_TYPE);

            AddProjectResult addProjectResult = new AddProjectResult();
            addProjectResult.setResult(true);
            addProjectResult.setDbName(project.getDbName());
            addProjectResult.setProjectID(project.getID());
            return addProjectResult;
        } catch (Exception e) {
            operationLogService.addDataAddLog(null, OperationModule.Project, userName,
                    Message.Project.AddProjectFail, e.getMessage(), projectDto.getName(), LOG_TYPE);

            AddProjectResult addProjectResult = new AddProjectResult();
            addProjectResult.setResult(false);
            addProjectResult.setResultMsg(e.getMessage());
            return addProjectResult;
        }

    }

    @Override
    public Map<Integer, Integer> getProjectAllStatus(String dbName) {
        List<ProjectStatusManage> psmList = projectMapper.selectProjectAllStatus(dbName);
        Map<Integer, Integer> result = new HashMap<>(psmList.size());
        psmList.forEach(m -> {
            result.put(m.getPeriodId(), m.getStatus());
        });

        return result;
    }

    static class DbName {
        String dbName;

        public DbName(String dbName) {
            this.dbName = dbName;
        }
    }

    private String generateDatabase(Project project, String accsetid) {//TODO: should be impl future(neo)
        String maxDbName = projectMapper.maxDbName(project.getYear());
        String dbName = "";

        if (!Utils.isEmpty(maxDbName)) {
            dbName = String.format("%s%s%s", "a", (project.getYear() + "").substring(2), "00000");
        } else {
            dbName = generateProjectDbName(dbName, project.getYear() + "");
        }
        DbName sbDbName = new DbName(dbName);

        createDatabaseByName(sbDbName, project);
        ShardingContextHolder.setDataSourceKey(sbDbName.dbName);

        try (SqlSession sqlSession = dynamicSqlSessionTemplate.getSqlSessionFactory().openSession();
             BufferedReader bufferedReader = new BufferedReader(prepareRunScriptReader(DB_MYSQL_PATH, sbDbName.dbName,
                     project, accsetid));) {

            ScriptRunner scriptRunner = new ScriptRunner(sqlSession.getConnection());//only support mysql
            scriptRunner.setAutoCommit(true);
            scriptRunner.runScript(bufferedReader);
        } catch (IOException e) {
            e.printStackTrace();
            LOGGER.error("generate data base error", e);
        }

        initStandardAccount(project.getIndustryID());
        return sbDbName.dbName;

    }

    private void initStandardAccount(String industryId) {
        StandardAccountExample example = new StandardAccountExample();
        example.createCriteria().andIsActiveEqualTo(ACTIVE).andIndustryIDIn(Lists.newArrayList(industryId, DEFAULT_INDUSTRY_ID));
        List<StandardAccount> standardAccountList = standardAccountMapper.selectByExample(example);

        List<VatStandardAccount> prepareAccount = new ArrayList<>();

        if (industryId.equalsIgnoreCase(DEFAULT_INDUSTRY_ID)) {
            prepareAccount = extractFrom(standardAccountList);
        } else {
            final List<StandardAccount> groupDefault = new ArrayList<>();
            final List<StandardAccount> groupInput = new ArrayList<>();

            standardAccountList.forEach(m -> {
                if (m.getIndustryID().equalsIgnoreCase(industryId)) groupInput.add(m);
                else groupDefault.add(m);
            });

            if (groupInput.isEmpty()) {
                if (!groupDefault.isEmpty())
                    prepareAccount = extractFrom(groupDefault);
            } else {
                prepareAccount = extractFrom(groupInput);
            }
        }

        prepareAccount.forEach(m -> vatStandardAccountMapper.insert(m));

    }

    private List<VatStandardAccount> extractFrom(List<StandardAccount> fromSource) {
        List<VatStandardAccount> resut = new ArrayList<>(fromSource.size());
        fromSource.forEach(m -> {
            VatStandardAccount dest = new VatStandardAccount();
            try {
                FieldsMapper.map(m, dest);
            } catch (Exception e) {
                LOGGER.warn("some filed can not mapped", e);
            }
            resut.add(dest);
        });

        return resut;
    }

    private Reader prepareRunScriptReader(String sqlPath, String dbName, Project project, String accsetid) {
        Map<String, String> keyValues = new HashMap<>();
        keyValues.put("DBKeyword_ProjectDbName", dbName);
        keyValues.put("DBKeyword_OrganizationID", project.getOrganizationID());
        keyValues.put("DBKeyword_EnterpriseAccountSetID", accsetid);
        keyValues.put("DBKeyword_AdminDbName", adminDbName);

        BufferedReader bufferedReader = null;
        String line = null;
        StringWriter sw = new StringWriter();
        BufferedWriter writer = new BufferedWriter(sw);
        String resultContent = null;
        try {
            Resources.setCharset(Charset.forName("UTF-8"));
            bufferedReader = new BufferedReader(Resources.getResourceAsReader(sqlPath));

            while ((line = bufferedReader.readLine()) != null) {
                Iterator<Map.Entry<String, String>> kv = keyValues.entrySet().iterator();
                while (kv.hasNext()) {
                    Map.Entry<String, String> entry = kv.next();
                    line = line.replaceAll(entry.getKey(), entry.getValue());
                }

                writer.write(line);
                writer.newLine();
            }

            writer.flush();
            resultContent = sw.getBuffer().toString();
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.warn("prepare run reader some error", e);
        } finally {
            try {
                if (writer != null) writer.close();
                if (sw != null) sw.close();
                if (bufferedReader != null) bufferedReader.close();

            } catch (Exception e) {
                LOGGER.error("io resource close some error", e);
            }

        }

        return new StringReader(resultContent);

    }

    private void createDatabaseByName(DbName dbName, Project project) {
        int result = databaseAgent.createDatabase(dbName.dbName);

        if (result == Constant.CREATE_DB_EXISTS) {
            dbName.dbName = generateProjectDbName(dbName.dbName, project.getYear() + "");
            createDatabaseByName(dbName, project);
        } else if (result == Constant.CREATE_DB_FAILED) {
            LOGGER.debug("should throws biz exception");//TODO: should shrow biz exception in futrue(neo)
        }

    }


    private String generateProjectDbName(String maxName, String year) {
        year = year.substring(2);
        String pre2 = maxName.charAt(0) + "", post2 = "";
        int post = Integer.parseInt(maxName.substring(3)) + 1;
        if (post >= 100000) {
            Integer pre = Integer.parseInt(String.valueOf(maxName.charAt(0)));
            pre2 = ((char) (pre + 1)) + "";
            post2 = "00000";
        } else if (post >= 10000) {
            post2 = post + "";
        } else if (post >= 1000) {
            post2 = "0" + post;
        } else if (post >= 100) {
            post2 = "00" + post;
        } else if (post >= 10) {
            post2 = "000" + post;
        } else {
            post2 = "0000" + post;
        }
        return pre2 + year + post2;
    }

    private List<ProjectDisplayDto> getProjectFromEnterpriseAccountSetOrg(List<ProjectDisplayDto> pList, String orgID,
                                                                          String serviceID, Integer projectYear) {
        List<ProjectDisplayDto> list = new ArrayList<>();
        List<ProjectDisplayDto> data = projectMapper.getProjectFromEnterpriseAccountSetOrg(orgID, serviceID);
        Map<String, ProjectDisplayDto> groupMap = new HashMap<>();

        if (data != null && data.size() > 0) {
            //去除project表已经存在的项目 g
            data.forEach(dto -> {
                getOrgProjectYears(dto.getEffectiveDate(), dto.getExpiredDate()).forEach(yDto -> {
                    dto.setYear(yDto.year);
                    dto.setStartPeriod(yDto.startMonth);
                    dto.setEndPeriod(yDto.endMonth);
                    dto.setName(dto.getOrganizationName() + "(" + dto.getYear() + "）");

                    //判断是否已经创建过该项目
                    //对于VAT，只创建今年及之后年份的项目(2018.1.2经Michelle确定，已放开vat限制)
                    //if ((projectList != null && projectList.Count > 0) || (int.Parse(orgDto.ServiceTypeID) == 2 && orgDto.Year < DateTime.Now.Year))
                    if (!pList.stream().anyMatch((Predicate<ProjectDisplayDto>) t -> t.getYear().equals(dto.getYear())
                            && t.getServiceTypeID().equals(dto.getServiceTypeID())
                            && t.getOrganizationID().equals(dto.getOrganizationID())
                            && t.getEnterpriseAccountSetID().equals(dto.getEnterpriseAccountSetID())
                            && t.getStartPeriod().equals(dto.getStartPeriod())
                            && t.getEndPeriod().equals(dto.getEndPeriod()))) {
                        list.add(dto);
                    }
                });
            });

            if (!list.isEmpty()) {
                list.forEach(m -> {
                    StringBuilder groupKeyBulder = new StringBuilder(m.getOrganizationID()).append("_")
                            .append(m.getEnterpriseAccountSetID()).append("_")
                            .append(m.getYear()).append("_")
                            .append(m.getServiceTypeID()).append("_")
                            .append(m.getStartPeriod()).append("_")
                            .append(m.getEndPeriod());

                    groupMap.putIfAbsent(groupKeyBulder.toString(), m);
                });
            }
        }

        return Lists.newArrayList(groupMap.values());

    }

    private List<ProjectYearParam> getOrgProjectYears(Date startDate, Date endDate) {
        List<ProjectYearParam> pList = new ArrayList<>();

        ProjectYearParam pYear = new ProjectYearParam();
        Calendar nowCal = Calendar.getInstance();
        Date nowDate = new Date();
        nowCal.setTime(nowDate);

        Calendar startCal = Calendar.getInstance();
        startCal.setTime(startDate);

        Calendar endCal = Calendar.getInstance();
        endCal.setTime(endDate);
        if (startDate == null && endDate == null) {
            pYear.year = nowCal.get(Calendar.YEAR);
            pYear.startMonth = 1;
            pYear.endMonth = 12;

            pList.add(pYear);
        } else if (startDate == null && endDate != null && endCal.after(nowCal)) {
            pList.addAll(getProjectYears(nowDate, endDate));
        } else if (startDate != null && endDate == null && startCal.before(nowCal)) {
            pList.addAll(getProjectYears(startDate, nowDate));
        } else if (startDate != null && endDate != null && startCal.get(Calendar.YEAR) <= endCal.get(Calendar.YEAR)) {
            pList.addAll(getProjectYears(startDate, endDate));
        }
        return pList;
    }

    private List<ProjectYearParam> getProjectYears(Date startDate, Date endDate) {
        List<ProjectYearParam> pList = new ArrayList<>();

        Calendar start = Calendar.getInstance();
        start.setTime(startDate);
        int startYear = start.get(Calendar.YEAR);

        Calendar end = Calendar.getInstance();
        end.setTime(endDate);
        int endYear = end.get(Calendar.YEAR);

        if (startDate != null && endDate != null && startYear <= endYear) {
            for (int i = startYear; i <= endYear; i++) {
                ProjectYearParam pYear = new ProjectYearParam();
                pYear.year = i;
                if (startYear == endYear) {
                    pYear.startMonth = start.get(Calendar.MONTH);
                    pYear.endMonth = end.get(Calendar.MONTH);
                } else if (i == startYear && i < endYear) {
                    pYear.startMonth = start.get(Calendar.MONTH);
                    pYear.endMonth = 12;
                } else if (i > startYear && i < endYear) {
                    pYear.startMonth = 1;
                    pYear.endMonth = 12;
                } else if (i > startYear && i == endYear) {
                    pYear.startMonth = 1;
                    pYear.endMonth = end.get(Calendar.MONTH);
                }
                pList.add(pYear);
            }
        }
        return pList;
    }

    /**
     * @TODO: change the return part after copyProperties function updated
     */
    private ServiceTypeDto rotateServiceType(ServiceType serviceType) {
        ServiceTypeDto serviceTypeDto = new ServiceTypeDto();
        return CommonUtils.copyProperties(serviceType, serviceTypeDto);
    }

    /**
     * @TODO: change the return part after copyProperties function updated
     */
    private TemplateGroupDto rotateTemplateGroup(TemplateGroup templateGroup) {
        TemplateGroupDto templateGroupDto = new TemplateGroupDto();
        return CommonUtils.copyProperties(templateGroup, templateGroupDto);
    }

    /**
     * @TODO: change the return part after copyProperties function updated
     */
    private ProjectClientDto rotateProjectClient(ProjectClient projectClient) {
        ProjectClientDto projectClientDto = new ProjectClientDto();
        return CommonUtils.copyProperties(projectClient, projectClientDto);
    }

}
