package pwc.taxtech.atms.vat.service.impl;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.OperandResolver;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.dto.GroupId;
import pwc.taxtech.atms.dto.vatdto.CellTemplatePerGroupDto;
import pwc.taxtech.atms.entitiy.ProjectServiceType;
import pwc.taxtech.atms.entitiy.ProjectServiceTypeExample;
import pwc.taxtech.atms.vat.service.impl.report.functions.FormulaAgent;
import pwc.taxtech.atms.vat.service.impl.report.functions.FormulaContext;

import java.util.Comparator;
import java.util.List;

public class ReportFormulaFactory {
    static final Logger LOGGER = LoggerFactory.getLogger(ReportFormulaFactory.class);

    public static abstract class ReportBaseFormula implements FreeRefFunction {
        final FormulaContext context;
        final FormulaAgent agent;

        protected ReportBaseFormula(FormulaContext context) {
            this.context = context;
            this.agent = context.getFormulaAgent();
        }

        static String resolverString(ValueEval[] args, OperationEvaluationContext ec, int index)
                throws EvaluationException {
            return OperandResolver.coerceValueToString(OperandResolver.getSingleValue(args[index], ec.getRowIndex(),
                    ec.getColumnIndex()));
        }

        static Integer resolverInteger(ValueEval[] args, OperationEvaluationContext ec, int index)
                throws EvaluationException {
            return OperandResolver.coerceValueToInt(OperandResolver.getSingleValue(args[index], ec.getRowIndex(),
                    ec.getColumnIndex()));
        }

        static Boolean resolverBoolean(ValueEval[] args, OperationEvaluationContext ec, int index)
                throws EvaluationException {
            return OperandResolver.coerceValueToBoolean(OperandResolver.getSingleValue(args[index], ec.getRowIndex(),
                    ec.getColumnIndex()), false);
        }


        static Double resolverDouble(ValueEval[] args, OperationEvaluationContext ec, int index)
                throws EvaluationException {
            return OperandResolver.coerceValueToDouble(OperandResolver.getSingleValue(args[index], ec.getRowIndex(),
                    ec.getColumnIndex()));
        }


        static class Period {
            int yearOffset = 0;
            int period;

            public Period(int period) {
                this.period = period;
            }

            public Period set(int yearOffset) {
                this.yearOffset = yearOffset;
                return this;
            }
        }

        Period getPeriod(int parameterPeriod) {
            int yearOffset = 0;
            if (parameterPeriod == -99) {
                return new Period(parameterPeriod);
            } else if (parameterPeriod <= -1) {
                Integer period = parameterPeriod + context.getPeriod();
                while (period <= 0) {
                    yearOffset--;
                    period += 12;
                    //throw new NotImplementedException();
                }

                return new Period(period).set(yearOffset);
            } else if (parameterPeriod == 0) {
                return new Period(context.getPeriod());
            } else {
                return new Period(parameterPeriod);
            }
        }

        public int getYear(int parameterYear) {
            if (parameterYear <= -1) {
                return parameterYear + context.getYear();
            } else if (parameterYear == 0) {
                return context.getYear();
            } else {
                return parameterYear;
            }
        }

    }

    public static class BB extends ReportBaseFormula {
        String reportCode;
        Integer columnIndex;
        Integer rowIndex;
        Integer period;
        Integer year;

        protected BB(FormulaContext context) {
            super(context);
        }

        void resolverArgs(ValueEval[] args, OperationEvaluationContext ec) throws EvaluationException {
            reportCode = resolverString(args, ec, 0);
            columnIndex = resolverInteger(args, ec, 1);
            rowIndex = resolverInteger(args, ec, 3);
            period = resolverInteger(args, ec, 4);
            year = resolverInteger(args, ec, 5);
        }


        @Override
        public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
            try {
                resolverArgs(args, ec);
                Period curPeriod = getPeriod(period.intValue());
                curPeriod.set(curPeriod.yearOffset + getYear(year));

                ProjectServiceTypeExample pst = new ProjectServiceTypeExample();
                pst.createCriteria().andTemplateGroupIDEqualTo(Long.valueOf(context.getReportTemplateGroupID()))
                        .andServiceTypeIDEqualTo(EnumServiceType.VAT.getCode() + "");
                List<ProjectServiceType> pstReult = agent.projectServiceTypeMapper.selectByExample(pst);

                List<CellTemplatePerGroupDto> cellTemplates = agent.adminMp.getCellTemplatePerGroupDto(context.getReportTemplateGroupID());
                if (pstReult == null || pstReult.isEmpty()) {
                    List<GroupId> groupIds = agent.adminMp.getTemplateGroupId(context.getProjectID());
                    GroupId groupId;

                    if (groupIds.size() > 0) {
                        groupId = groupIds.stream().filter(m -> m.isDefault != 1).findFirst().get();
                        if (groupId == null) groupId = groupIds.get(0);
                        cellTemplates.addAll(agent.adminMp.getCellTemplatePerGroupDto(groupId.groupId));
                    }
                }



            } catch (EvaluationException e) {
                LOGGER.warn("resole args some exception", e);
                e.printStackTrace();
            }

            return null;
        }

    }


}
