package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.dto.vatdto.BSPLFormulaDataSourceDto;
import pwc.taxtech.atms.dto.vatdto.FormulaDataSourceDto;
import pwc.taxtech.atms.entitiy.Project;
import pwc.taxtech.atms.entitiy.ProjectExample;
import pwc.taxtech.atms.vat.entity.BalanceStdManual;
import pwc.taxtech.atms.vat.entity.VatEnterpriseAccount;
import pwc.taxtech.atms.vat.entity.VatStandardAccount;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;


/// <summary>
/// 名称：发生净值
/// 功能：计算指定科目发生额（根据科目自带方向而定）
/// 备注：科目在贷方 则用贷方发生额-借方发生额
///       科目在借方 则用借方发生额-贷方发生额
///       本公式取VAT试算平衡表 总账（经过科目重分类和审计调整之后的）中累计发生额相关数据。
/// web admin中需要新增科目(科目代码待确定)，6001.04 税率为17%， 6001.05税率为13%，6001.06税率为11%，6001.07税率为6%
/// </summary>
/// <param name="accountType">科目类型——两种取值结果“0”标准科目，“1”企业科目；</param>
/// <param name="accountCode">科目代码；</param>
/// <param name="year">会计年份——“0”为当前年份，其他年份则为具体年份；</param>
/// <param name="period">会计期间——“0”为当前月份，1-12为自然月；"99"当年累计</param>
/// <param name="keyParam">摘要（未实现）</param>
public class FSJZ extends FunctionBase implements FreeRefFunction {

    private FormulaContext formulaContext;

    public FSJZ(FormulaContext formulaContext) {
        this.formulaContext = formulaContext;
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        int accountType, year, period;
        String accountCode, keyParam;

        if (args.length < 5) {
            return null;
        }

        accountType = this.getIntParam(args[0], ec);
        accountCode = this.getStringParam(args[1], ec);
        year = this.getIntParam(args[2], ec);
        period = this.getIntParam(args[3], ec);
        keyParam = this.getStringParam(args[4], ec);

        if (year == 0) {
            year = formulaContext.getYear();
        }

        int[] yearOffset = {};
        period = FormulaHelper.getPeriod(period, yearOffset, formulaContext);
        year = year + yearOffset[0];

        if (accountType == 0) {
            VatStandardAccount vatStandardAccount = formulaContext.getFormulaAgent().vatStandardAccountMapper.selectOneByAccountCode(accountCode);
            if (vatStandardAccount == null) {
                return null;
            }

            BalanceStdManual balanceStdManual = null;
            if (year != formulaContext.getYear()) {
                ProjectExample example = new ProjectExample();
                example.createCriteria().andOrganizationIDEqualTo(formulaContext.getOrganizationID()).andYearEqualTo(year);
                Project project = formulaContext.getFormulaAgent().projectMapper.selectByExample(example).get(0);
                String dbName = StringUtils.EMPTY;
                if (project == null) {
                    return null;
                } else {
                    dbName = project.getDbName();
                }

                if (period == 99) {
                    balanceStdManual = formulaContext.getFormulaAgent().balanceStdManualMapper.selectOneWithAccountCodeAndDBName(accountCode, dbName, 12);
                } else {
                    balanceStdManual = formulaContext.getFormulaAgent().balanceStdManualMapper.selectOneWithAccountCodeAndDBName(accountCode, dbName, period);
                }
            } else {
                if (period == 99) {
                    balanceStdManual = formulaContext.getFormulaAgent().balanceStdManualMapper.selectOneWithAccountCode(accountCode, 12);
                } else {
                    balanceStdManual = formulaContext.getFormulaAgent().balanceStdManualMapper.selectOneWithAccountCode(accountCode, period);
                }
            }

            BigDecimal val = new BigDecimal(0);
            List<FormulaDataSourceDto> formulaDataSourceDtoList = new ArrayList<>();
            if (balanceStdManual != null) {
                if (period == 99) {
                    val = new BigDecimal(((balanceStdManual.getYearDebitNet() != null ? balanceStdManual.getYearDebitNet() : 0).doubleValue()
                            - (balanceStdManual.getYearCreditNet() != null ? balanceStdManual.getYearCreditNet() : 0).doubleValue())
                            * vatStandardAccount.getDirection());
                    List<VatEnterpriseAccount> vatEnterpriseAccountList = formulaContext.getFormulaAgent().vatEnterpriseAccountMapper.getListWithAccountCode(accountCode);
                    for(VatEnterpriseAccount vatEnterpriseAccount:vatEnterpriseAccountList){
                        BigDecimal amount = formulaContext.getFormulaAgent().balanceMapper.getSumOfBalance(vatEnterpriseAccount.getAcctCode(),vatStandardAccount.getDirection());
                        BSPLFormulaDataSourceDto bsplFormulaDataSourceDto = new BSPLFormulaDataSourceDto();
                        bsplFormulaDataSourceDto.setAccountCode(vatEnterpriseAccount.getAcctCode());
                        bsplFormulaDataSourceDto.setAccountName(vatEnterpriseAccount.getName());
                        bsplFormulaDataSourceDto.setName("TrialBalanceDataSource");
                        bsplFormulaDataSourceDto.setAmount(amount);
                        formulaDataSourceDtoList.add(bsplFormulaDataSourceDto);
                    }
                } else {

                }
            }


        } else {

        }
        return null;
    }
}
