package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.OperandResolver;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.dto.vatdto.CellTemplatePerGroupDto;

import java.util.List;
import java.util.Optional;



public class SGSR implements FreeRefFunction {
    private FormulaContext formulaContext;

    public SGSR(FormulaContext formulaContext) {
        this.formulaContext = formulaContext;
    }

    @Override
    public ValueEval evaluate(ValueEval[] valueEvals, OperationEvaluationContext operationEvaluationContext) {
        try {
            int period, year, colIndex, rowIndex;
            String reportCode;
            if (valueEvals.length > 4) {
                ValueEval v5 = OperandResolver.getSingleValue(valueEvals[4],
                        operationEvaluationContext.getRowIndex(),
                        operationEvaluationContext.getColumnIndex());
                year = OperandResolver.coerceValueToInt(v5);
            } else if (valueEvals.length == 4) {
                year = 0;
            } else {
                return null;
            }

            ValueEval v1 = OperandResolver.getSingleValue(valueEvals[0],
                    operationEvaluationContext.getRowIndex(),
                    operationEvaluationContext.getColumnIndex());
            reportCode = OperandResolver.coerceValueToString(v1);

            ValueEval v2 = OperandResolver.getSingleValue(valueEvals[1],
                    operationEvaluationContext.getRowIndex(),
                    operationEvaluationContext.getColumnIndex());
            String cIndex = OperandResolver.coerceValueToString(v2);
            colIndex = FormulaHelper.excelColStrToNum(cIndex, cIndex.length());


            ValueEval v3 = OperandResolver.getSingleValue(valueEvals[2],
                    operationEvaluationContext.getRowIndex(),
                    operationEvaluationContext.getColumnIndex());
            rowIndex = OperandResolver.coerceValueToInt(v3);

            ValueEval v4 = OperandResolver.getSingleValue(valueEvals[3],
                    operationEvaluationContext.getRowIndex(),
                    operationEvaluationContext.getColumnIndex());
            period = OperandResolver.coerceValueToInt(v4);

            int curYear, curPeriod;
            if (reportCode.startsWith("VAT")) {
                int[] yearOffSet = {};
                curPeriod = FormulaHelper.getPeriod(period, yearOffSet, formulaContext);
                curYear = FormulaHelper.getYear(year, formulaContext) + yearOffSet[0];
            } else if (reportCode.startsWith("CIT.")) {
                curYear = FormulaHelper.getYear(year, formulaContext);
                curPeriod = 0;
            } else {
                return null;
            }

            List<CellTemplatePerGroupDto> cellTemplatePerGroupDtoList = formulaContext.getFormulaAgent().periodTemplateMapper
                    .getCellTemplatePerGroup(formulaContext.getPeriod(), formulaContext.getReportTemplateGroupID());

            int r = rowIndex;
            int c = colIndex;

            Optional<CellTemplatePerGroupDto> cellTemplatePerGroupDto = cellTemplatePerGroupDtoList.stream()
                    .filter(a -> a.getRowIndex() == r && a.getColumnIndex() == c && a.getReportCode().equals(reportCode))
                    .findFirst();

            if (cellTemplatePerGroupDto == null) {
                return null;
            }

            //配置的行号列号都比 DB 中+1
            rowIndex--;
            colIndex--;



        } catch (EvaluationException e) {
            e.printStackTrace();
        }
        return null;
    }
}
